@@ -2,7 +2,10 @@
         pkgList <- c("filehash", "stashR")
         
         for(pkg in pkgList) {
-                if(!require(pkg, quietly = TRUE, character.only = TRUE))
+                status <- suppressMessages({
+                        require(pkg, quietly = TRUE, character.only = TRUE)
+                })
+                if(!status)
                         stop(gettextf("'%s' package required", pkg))
         }
         assign("cacheDir", ".", .cacheEnv)
